@@ -2,10 +2,10 @@
  *
  * rdf_node.c - RDF Node (RDF URI, Literal, Blank Node) Interface
  *
- * $Id: rdf_node.c 12178 2007-05-01 03:32:14Z dajobe $
+ * $Id: rdf_node.c 13287 2007-12-20 12:00:50Z laalto $
  *
- * Copyright (C) 2000-2006, David Beckett http://purl.org/net/dajobe/
- * Copyright (C) 2000-2005, Copyright (C) 2000-2006, Copyright (C) 2000-2006, University of Bristol, UK http://www.bristol.ac.uk/
+ * Copyright (C) 2000-2007, David Beckett http://purl.org/net/dajobe/
+ * Copyright (C) 2000-2005, University of Bristol, UK http://www.bristol.ac.uk/
  * 
  * This package is Free Software and part of Redland http://librdf.org/
  * 
@@ -392,7 +392,7 @@ librdf_new_node_from_literal(librdf_world *world,
                                                     xml_language,
                                                     xml_language_len,
                                                     (is_wf_xml ? 
-                                                     LIBRDF_RS_XMLLiteral_URI : NULL)
+                                                     LIBRDF_RS_XMLLiteral_URI(world) : NULL)
                                                     );
 }
 
@@ -462,7 +462,8 @@ librdf_new_node_from_typed_counted_literal(librdf_world *world,
     new_node=NULL;
     goto unlock;
   }
-  strcpy((char*)new_value, (const char*)value);
+  strncpy((char*)new_value, (const char*)value, value_len);
+  new_value[value_len]='\0';
   new_node->value.literal.string=new_value;
   
   if(xml_language && *xml_language) {
@@ -473,7 +474,8 @@ librdf_new_node_from_typed_counted_literal(librdf_world *world,
       new_node=NULL;
       goto unlock;
     }
-    strcpy(new_xml_language, xml_language);
+    strncpy(new_xml_language, xml_language, xml_language_len);
+    new_xml_language[xml_language_len]='\0';
     new_node->value.literal.xml_language=new_xml_language;
     new_node->value.literal.xml_language_len=xml_language_len;
   } else
@@ -492,6 +494,8 @@ librdf_new_node_from_typed_counted_literal(librdf_world *world,
     buffer=NULL;
   
   if(!buffer) {
+    if(new_xml_language)
+      LIBRDF_FREE(cstring, new_xml_language);
     if(datatype_uri)
       librdf_free_uri(datatype_uri);
     LIBRDF_FREE(cstring, new_value);
@@ -619,18 +623,23 @@ librdf_new_node_from_blank_identifier(librdf_world *world,
   pthread_mutex_lock(world->nodes_mutex);
 #endif
 
-  if(!identifier)
-    identifier=librdf_world_get_genid(world);
-
-  len=strlen((const char*)identifier);
-  
-  new_identifier=(unsigned char*)LIBRDF_MALLOC(cstring, len+1);
-  if(!new_identifier) {
-    new_node=NULL;
-    goto unlock;
+  if(!identifier) {
+    new_identifier=librdf_world_get_genid(world);
+    if(!new_identifier) {
+      new_node=NULL;
+      goto unlock;
+    }
+    len=strlen((const char *)new_identifier);
+  } else {
+    len=strlen((const char*)identifier);
+    
+    new_identifier=(unsigned char*)LIBRDF_MALLOC(cstring, len+1);
+    if(!new_identifier) {
+      new_node=NULL;
+      goto unlock;
+    }
+    strcpy((char*)new_identifier, (const char*)identifier);
   }
-  strcpy((char*)new_identifier, (const char*)identifier);
-
 
   key.data=new_identifier;
   key.size=len;
@@ -746,9 +755,9 @@ librdf_free_node(librdf_node *node)
     case LIBRDF_NODE_TYPE_RESOURCE:
       key.data=&node->value.resource.uri;
       key.size=sizeof(librdf_uri*);
-      if(librdf_hash_delete_all(node->world->nodes_hash[H_RESOURCE], &key) )
-        LIBRDF_FATAL1(node->world, LIBRDF_FROM_NODE, "Hash deletion failed");
-
+      /* Hash deletion fails only if the key is not found.
+         This is not a fatal error so do not check for return value. */
+      librdf_hash_delete_all(node->world->nodes_hash[H_RESOURCE], &key);
       librdf_free_uri(node->value.resource.uri);
       break;
       
@@ -756,8 +765,7 @@ librdf_free_node(librdf_node *node)
       if(node->value.literal.key) {
         key.data=node->value.literal.key;
         key.size=node->value.literal.size;
-        if(librdf_hash_delete_all(node->world->nodes_hash[H_LITERAL], &key) )
-          LIBRDF_FATAL1(node->world, LIBRDF_FROM_NODE, "Hash deletion failed");
+        librdf_hash_delete_all(node->world->nodes_hash[H_LITERAL], &key); /* see above */
         LIBRDF_FREE(cstring, node->value.literal.key);
       }
       
@@ -772,9 +780,7 @@ librdf_free_node(librdf_node *node)
     case LIBRDF_NODE_TYPE_BLANK:
       key.data=node->value.blank.identifier;
       key.size=node->value.blank.identifier_len;
-      if(librdf_hash_delete_all(node->world->nodes_hash[H_BLANK], &key) )
-        LIBRDF_FATAL1(node->world, LIBRDF_FROM_NODE, "Hash deletion failed");
-
+      librdf_hash_delete_all(node->world->nodes_hash[H_BLANK], &key); /* see above */
       if(node->value.blank.identifier != NULL)
         LIBRDF_FREE(cstring, node->value.blank.identifier);
       break;
@@ -983,7 +989,7 @@ librdf_node_get_literal_value_is_wf_xml(librdf_node* node)
     return 0;
   
   return librdf_uri_equals(node->value.literal.datatype_uri,
-                           LIBRDF_RS_XMLLiteral_URI);
+                           LIBRDF_RS_XMLLiteral_URI(node->world));
 }
 
 
@@ -1553,7 +1559,7 @@ librdf_node_decode(librdf_world *world,
                                                       string_length,
                                                       (const char*)language,
                                                       language_length,
-                                                      is_wf_xml ? LIBRDF_RS_XMLLiteral_URI : NULL);
+                                                      is_wf_xml ? LIBRDF_RS_XMLLiteral_URI(world) : NULL);
     
     break;
 
@@ -1742,7 +1748,7 @@ librdf_node_static_iterator_create(librdf_node** nodes,
                                    int size)
 {
   librdf_node_static_iterator_context* context;
-  librdf_iterator *iterator;
+  librdf_iterator* iterator;
   
   LIBRDF_ASSERT_OBJECT_POINTER_RETURN_VALUE(nodes, librdf_node**, NULL);
 
@@ -1762,7 +1768,6 @@ librdf_node_static_iterator_create(librdf_node** nodes,
                                librdf_node_static_iterator_finished);
   if(!iterator)
     librdf_node_static_iterator_finished(context);
-
   return iterator;
 }
 
